"use strict";
// Source file from duniter: Crypto-currency software to manage libre currency such as Ğ1
// Copyright (C) 2018  Cedric Moreau <cem.moreau@gmail.com>
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
Object.defineProperty(exports, "__esModule", { value: true });
const crypto = require("crypto");
const lib_1 = require("../../../../neon/lib");
const SEED_LENGTH = 32; // Length of the key
/**
 * Generates a new keypair object from salt + password strings.
 * @param salt
 * @param key
 * @param N Scrypt parameter N. Defaults to 4096.
 * @param r Scrypt parameter r. Defaults to 16.
 * @param p Scrypt parameter p. Defaults to 1.
 * @return keyPair An object containing the public and private keys, base58 encoded.
 */
exports.Scrypt = async (salt, key, N = 4096, r = 16, p = 1) => {
    const res = await new Promise((resolve, reject) => {
        crypto.scrypt(key, salt, SEED_LENGTH, { N, r, p }, (err, seed) => {
            if (err)
                return reject(err);
            const pair = lib_1.KeyPairBuilder.fromSeed(seed);
            resolve({
                pub: pair.getPublicKey(),
                sec: lib_1.seedToSecretKey(seed),
            });
        });
    });
    return res;
};
//# sourceMappingURL=scrypt.js.map